/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeinputdevicemodel_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/systeminfo/qdeclarativeinputdevicemodel_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeinputdevicemodel_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeInputDeviceModel_t {
    QByteArrayData data[32];
    char stringdata0[367];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeInputDeviceModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeInputDeviceModel_t qt_meta_stringdata_QDeclarativeInputDeviceModel = {
    {
QT_MOC_LITERAL(0, 0, 28), // "QDeclarativeInputDeviceModel"
QT_MOC_LITERAL(1, 29, 5), // "added"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 13), // "QInputDevice*"
QT_MOC_LITERAL(4, 50, 11), // "inputDevice"
QT_MOC_LITERAL(5, 62, 7), // "removed"
QT_MOC_LITERAL(6, 70, 8), // "deviceId"
QT_MOC_LITERAL(7, 79, 13), // "filterChanged"
QT_MOC_LITERAL(8, 93, 28), // "QInputDevice::InputTypeFlags"
QT_MOC_LITERAL(9, 122, 11), // "filterFlags"
QT_MOC_LITERAL(10, 134, 12), // "countChanged"
QT_MOC_LITERAL(11, 147, 7), // "devices"
QT_MOC_LITERAL(12, 155, 16), // "updateDeviceList"
QT_MOC_LITERAL(13, 172, 11), // "addedDevice"
QT_MOC_LITERAL(14, 184, 6), // "device"
QT_MOC_LITERAL(15, 191, 13), // "removedDevice"
QT_MOC_LITERAL(16, 205, 4), // "path"
QT_MOC_LITERAL(17, 210, 7), // "indexOf"
QT_MOC_LITERAL(18, 218, 10), // "devicePath"
QT_MOC_LITERAL(19, 229, 3), // "get"
QT_MOC_LITERAL(20, 233, 5), // "index"
QT_MOC_LITERAL(21, 239, 6), // "filter"
QT_MOC_LITERAL(22, 246, 5), // "count"
QT_MOC_LITERAL(23, 252, 9), // "ItemRoles"
QT_MOC_LITERAL(24, 262, 11), // "ServiceRole"
QT_MOC_LITERAL(25, 274, 8), // "NameRole"
QT_MOC_LITERAL(26, 283, 14), // "IdentifierRole"
QT_MOC_LITERAL(27, 298, 11), // "ButtonsRole"
QT_MOC_LITERAL(28, 310, 12), // "SwitchesRole"
QT_MOC_LITERAL(29, 323, 16), // "RelativeAxesRole"
QT_MOC_LITERAL(30, 340, 16), // "AbsoluteAxesRole"
QT_MOC_LITERAL(31, 357, 9) // "TypesRole"

    },
    "QDeclarativeInputDeviceModel\0added\0\0"
    "QInputDevice*\0inputDevice\0removed\0"
    "deviceId\0filterChanged\0"
    "QInputDevice::InputTypeFlags\0filterFlags\0"
    "countChanged\0devices\0updateDeviceList\0"
    "addedDevice\0device\0removedDevice\0path\0"
    "indexOf\0devicePath\0get\0index\0filter\0"
    "count\0ItemRoles\0ServiceRole\0NameRole\0"
    "IdentifierRole\0ButtonsRole\0SwitchesRole\0"
    "RelativeAxesRole\0AbsoluteAxesRole\0"
    "TypesRole"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeInputDeviceModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       2,   84, // properties
       1,   92, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x06 /* Public */,
       5,    1,   62,    2, 0x06 /* Public */,
       7,    1,   65,    2, 0x06 /* Public */,
      10,    1,   68,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    0,   71,    2, 0x0a /* Public */,
      13,    1,   72,    2, 0x08 /* Private */,
      15,    1,   75,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      17,    1,   78,    2, 0x02 /* Public */,
      19,    1,   81,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::Int,   11,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 3,   14,
    QMetaType::Void, QMetaType::QString,   16,

 // methods: parameters
    QMetaType::Int, QMetaType::QString,   18,
    0x80000000 | 3, QMetaType::Int,   20,

 // properties: name, type, flags
      21, 0x80000000 | 8, 0x0049510b,
      22, QMetaType::Int, 0x00495001,

 // properties: notify_signal_id
       2,
       3,

 // enums: name, alias, flags, count, data
      23,   23, 0x0,    8,   97,

 // enum data: key, value
      24, uint(QDeclarativeInputDeviceModel::ServiceRole),
      25, uint(QDeclarativeInputDeviceModel::NameRole),
      26, uint(QDeclarativeInputDeviceModel::IdentifierRole),
      27, uint(QDeclarativeInputDeviceModel::ButtonsRole),
      28, uint(QDeclarativeInputDeviceModel::SwitchesRole),
      29, uint(QDeclarativeInputDeviceModel::RelativeAxesRole),
      30, uint(QDeclarativeInputDeviceModel::AbsoluteAxesRole),
      31, uint(QDeclarativeInputDeviceModel::TypesRole),

       0        // eod
};

void QDeclarativeInputDeviceModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeInputDeviceModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->added((*reinterpret_cast< QInputDevice*(*)>(_a[1]))); break;
        case 1: _t->removed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->filterChanged((*reinterpret_cast< QInputDevice::InputTypeFlags(*)>(_a[1]))); break;
        case 3: _t->countChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->updateDeviceList(); break;
        case 5: _t->addedDevice((*reinterpret_cast< QInputDevice*(*)>(_a[1]))); break;
        case 6: _t->removedDevice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: { int _r = _t->indexOf((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 8: { QInputDevice* _r = _t->get((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QInputDevice**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInputDevice* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInputDevice* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeInputDeviceModel::*)(QInputDevice * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeInputDeviceModel::added)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeInputDeviceModel::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeInputDeviceModel::removed)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeInputDeviceModel::*)(QInputDevice::InputTypeFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeInputDeviceModel::filterChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeInputDeviceModel::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeInputDeviceModel::countChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeInputDeviceModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QInputDevice::InputTypeFlags*>(_v) = _t->filter(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->rowCount(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeInputDeviceModel *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFilter(*reinterpret_cast< QInputDevice::InputTypeFlags*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QDeclarativeInputDeviceModel[] = {
    QMetaObject::SuperData::link<QInputDevice::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QDeclarativeInputDeviceModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeInputDeviceModel.data,
    qt_meta_data_QDeclarativeInputDeviceModel,
    qt_static_metacall,
    qt_meta_extradata_QDeclarativeInputDeviceModel,
    nullptr
} };


const QMetaObject *QDeclarativeInputDeviceModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeInputDeviceModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeInputDeviceModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int QDeclarativeInputDeviceModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeInputDeviceModel::added(QInputDevice * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QDeclarativeInputDeviceModel::removed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QDeclarativeInputDeviceModel::filterChanged(QInputDevice::InputTypeFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeclarativeInputDeviceModel::countChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
