/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativedeviceinfo_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/systeminfo/qdeclarativedeviceinfo_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativedeviceinfo_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeDeviceInfo_t {
    QByteArrayData data[52];
    char stringdata0[682];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeDeviceInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeDeviceInfo_t qt_meta_stringdata_QDeclarativeDeviceInfo = {
    {
QT_MOC_LITERAL(0, 0, 22), // "QDeclarativeDeviceInfo"
QT_MOC_LITERAL(1, 23, 26), // "monitorThermalStateChanged"
QT_MOC_LITERAL(2, 50, 0), // ""
QT_MOC_LITERAL(3, 51, 21), // "activatedLocksChanged"
QT_MOC_LITERAL(4, 73, 19), // "enabledLocksChanged"
QT_MOC_LITERAL(5, 93, 19), // "thermalStateChanged"
QT_MOC_LITERAL(6, 113, 10), // "hasFeature"
QT_MOC_LITERAL(7, 124, 7), // "Feature"
QT_MOC_LITERAL(8, 132, 7), // "feature"
QT_MOC_LITERAL(9, 140, 9), // "imeiCount"
QT_MOC_LITERAL(10, 150, 4), // "imei"
QT_MOC_LITERAL(11, 155, 9), // "interface"
QT_MOC_LITERAL(12, 165, 12), // "manufacturer"
QT_MOC_LITERAL(13, 178, 5), // "model"
QT_MOC_LITERAL(14, 184, 11), // "productName"
QT_MOC_LITERAL(15, 196, 14), // "uniqueDeviceID"
QT_MOC_LITERAL(16, 211, 7), // "version"
QT_MOC_LITERAL(17, 219, 7), // "Version"
QT_MOC_LITERAL(18, 227, 4), // "type"
QT_MOC_LITERAL(19, 232, 19), // "monitorThermalState"
QT_MOC_LITERAL(20, 252, 14), // "activatedLocks"
QT_MOC_LITERAL(21, 267, 13), // "LockTypeFlags"
QT_MOC_LITERAL(22, 281, 12), // "enabledLocks"
QT_MOC_LITERAL(23, 294, 12), // "thermalState"
QT_MOC_LITERAL(24, 307, 12), // "ThermalState"
QT_MOC_LITERAL(25, 320, 16), // "BluetoothFeature"
QT_MOC_LITERAL(26, 337, 13), // "CameraFeature"
QT_MOC_LITERAL(27, 351, 14), // "FmRadioFeature"
QT_MOC_LITERAL(28, 366, 20), // "FmTransmitterFeature"
QT_MOC_LITERAL(29, 387, 15), // "InfraredFeature"
QT_MOC_LITERAL(30, 403, 10), // "LedFeature"
QT_MOC_LITERAL(31, 414, 17), // "MemoryCardFeature"
QT_MOC_LITERAL(32, 432, 10), // "UsbFeature"
QT_MOC_LITERAL(33, 443, 16), // "VibrationFeature"
QT_MOC_LITERAL(34, 460, 11), // "WlanFeature"
QT_MOC_LITERAL(35, 472, 10), // "SimFeature"
QT_MOC_LITERAL(36, 483, 18), // "PositioningFeature"
QT_MOC_LITERAL(37, 502, 15), // "VideoOutFeature"
QT_MOC_LITERAL(38, 518, 14), // "HapticsFeature"
QT_MOC_LITERAL(39, 533, 10), // "NfcFeature"
QT_MOC_LITERAL(40, 544, 8), // "LockType"
QT_MOC_LITERAL(41, 553, 6), // "NoLock"
QT_MOC_LITERAL(42, 560, 7), // "PinLock"
QT_MOC_LITERAL(43, 568, 19), // "TouchOrKeyboardLock"
QT_MOC_LITERAL(44, 588, 11), // "UnknownLock"
QT_MOC_LITERAL(45, 600, 14), // "UnknownThermal"
QT_MOC_LITERAL(46, 615, 13), // "NormalThermal"
QT_MOC_LITERAL(47, 629, 14), // "WarningThermal"
QT_MOC_LITERAL(48, 644, 12), // "AlertThermal"
QT_MOC_LITERAL(49, 657, 12), // "ErrorThermal"
QT_MOC_LITERAL(50, 670, 2), // "Os"
QT_MOC_LITERAL(51, 673, 8) // "Firmware"

    },
    "QDeclarativeDeviceInfo\0"
    "monitorThermalStateChanged\0\0"
    "activatedLocksChanged\0enabledLocksChanged\0"
    "thermalStateChanged\0hasFeature\0Feature\0"
    "feature\0imeiCount\0imei\0interface\0"
    "manufacturer\0model\0productName\0"
    "uniqueDeviceID\0version\0Version\0type\0"
    "monitorThermalState\0activatedLocks\0"
    "LockTypeFlags\0enabledLocks\0thermalState\0"
    "ThermalState\0BluetoothFeature\0"
    "CameraFeature\0FmRadioFeature\0"
    "FmTransmitterFeature\0InfraredFeature\0"
    "LedFeature\0MemoryCardFeature\0UsbFeature\0"
    "VibrationFeature\0WlanFeature\0SimFeature\0"
    "PositioningFeature\0VideoOutFeature\0"
    "HapticsFeature\0NfcFeature\0LockType\0"
    "NoLock\0PinLock\0TouchOrKeyboardLock\0"
    "UnknownLock\0UnknownThermal\0NormalThermal\0"
    "WarningThermal\0AlertThermal\0ErrorThermal\0"
    "Os\0Firmware"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeDeviceInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       4,   92, // properties
       5,  108, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    0,   75,    2, 0x06 /* Public */,
       4,    0,   76,    2, 0x06 /* Public */,
       5,    0,   77,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
       6,    1,   78,    2, 0x02 /* Public */,
       9,    0,   81,    2, 0x02 /* Public */,
      10,    1,   82,    2, 0x02 /* Public */,
      12,    0,   85,    2, 0x02 /* Public */,
      13,    0,   86,    2, 0x02 /* Public */,
      14,    0,   87,    2, 0x02 /* Public */,
      15,    0,   88,    2, 0x02 /* Public */,
      16,    1,   89,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // methods: parameters
    QMetaType::Bool, 0x80000000 | 7,    8,
    QMetaType::Int,
    QMetaType::QString, QMetaType::Int,   11,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString, 0x80000000 | 17,   18,

 // properties: name, type, flags
      19, QMetaType::Bool, 0x00495103,
      20, 0x80000000 | 21, 0x00495009,
      22, 0x80000000 | 21, 0x00495009,
      23, 0x80000000 | 24, 0x00495009,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,   15,  133,
      40,   40, 0x1,    4,  163,
      21,   40, 0x1,    4,  171,
      24,   24, 0x0,    5,  179,
      17,   17, 0x0,    2,  189,

 // enum data: key, value
      25, uint(QDeclarativeDeviceInfo::BluetoothFeature),
      26, uint(QDeclarativeDeviceInfo::CameraFeature),
      27, uint(QDeclarativeDeviceInfo::FmRadioFeature),
      28, uint(QDeclarativeDeviceInfo::FmTransmitterFeature),
      29, uint(QDeclarativeDeviceInfo::InfraredFeature),
      30, uint(QDeclarativeDeviceInfo::LedFeature),
      31, uint(QDeclarativeDeviceInfo::MemoryCardFeature),
      32, uint(QDeclarativeDeviceInfo::UsbFeature),
      33, uint(QDeclarativeDeviceInfo::VibrationFeature),
      34, uint(QDeclarativeDeviceInfo::WlanFeature),
      35, uint(QDeclarativeDeviceInfo::SimFeature),
      36, uint(QDeclarativeDeviceInfo::PositioningFeature),
      37, uint(QDeclarativeDeviceInfo::VideoOutFeature),
      38, uint(QDeclarativeDeviceInfo::HapticsFeature),
      39, uint(QDeclarativeDeviceInfo::NfcFeature),
      41, uint(QDeclarativeDeviceInfo::NoLock),
      42, uint(QDeclarativeDeviceInfo::PinLock),
      43, uint(QDeclarativeDeviceInfo::TouchOrKeyboardLock),
      44, uint(QDeclarativeDeviceInfo::UnknownLock),
      41, uint(QDeclarativeDeviceInfo::NoLock),
      42, uint(QDeclarativeDeviceInfo::PinLock),
      43, uint(QDeclarativeDeviceInfo::TouchOrKeyboardLock),
      44, uint(QDeclarativeDeviceInfo::UnknownLock),
      45, uint(QDeclarativeDeviceInfo::UnknownThermal),
      46, uint(QDeclarativeDeviceInfo::NormalThermal),
      47, uint(QDeclarativeDeviceInfo::WarningThermal),
      48, uint(QDeclarativeDeviceInfo::AlertThermal),
      49, uint(QDeclarativeDeviceInfo::ErrorThermal),
      50, uint(QDeclarativeDeviceInfo::Os),
      51, uint(QDeclarativeDeviceInfo::Firmware),

       0        // eod
};

void QDeclarativeDeviceInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeDeviceInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->monitorThermalStateChanged(); break;
        case 1: _t->activatedLocksChanged(); break;
        case 2: _t->enabledLocksChanged(); break;
        case 3: _t->thermalStateChanged(); break;
        case 4: { bool _r = _t->hasFeature((*reinterpret_cast< Feature(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->imeiCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->imei((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->manufacturer();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->model();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->productName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->uniqueDeviceID();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->version((*reinterpret_cast< Version(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeDeviceInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeDeviceInfo::monitorThermalStateChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeDeviceInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeDeviceInfo::activatedLocksChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeDeviceInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeDeviceInfo::enabledLocksChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeDeviceInfo::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeDeviceInfo::thermalStateChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeDeviceInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< bool*>(_v) = _t->monitorThermalState(); break;
        case 1: *reinterpret_cast<int*>(_v) = QFlag(_t->activatedLocks()); break;
        case 2: *reinterpret_cast<int*>(_v) = QFlag(_t->enabledLocks()); break;
        case 3: *reinterpret_cast< ThermalState*>(_v) = _t->thermalState(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeDeviceInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setMonitorThermalState(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeDeviceInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeDeviceInfo.data,
    qt_meta_data_QDeclarativeDeviceInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeDeviceInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeDeviceInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeDeviceInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeDeviceInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeDeviceInfo::monitorThermalStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeDeviceInfo::activatedLocksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeDeviceInfo::enabledLocksChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeDeviceInfo::thermalStateChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
