/****************************************************************************
** Meta object code from reading C++ file 'qdeviceinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/systeminfo/qdeviceinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeviceinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeviceInfo_t {
    QByteArrayData data[61];
    char stringdata0[819];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeviceInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeviceInfo_t qt_meta_stringdata_QDeviceInfo = {
    {
QT_MOC_LITERAL(0, 0, 11), // "QDeviceInfo"
QT_MOC_LITERAL(1, 12, 21), // "activatedLocksChanged"
QT_MOC_LITERAL(2, 34, 0), // ""
QT_MOC_LITERAL(3, 35, 26), // "QDeviceInfo::LockTypeFlags"
QT_MOC_LITERAL(4, 62, 5), // "types"
QT_MOC_LITERAL(5, 68, 19), // "enabledLocksChanged"
QT_MOC_LITERAL(6, 88, 19), // "thermalStateChanged"
QT_MOC_LITERAL(7, 108, 25), // "QDeviceInfo::ThermalState"
QT_MOC_LITERAL(8, 134, 5), // "state"
QT_MOC_LITERAL(9, 140, 21), // "bluetoothStateChanged"
QT_MOC_LITERAL(10, 162, 2), // "on"
QT_MOC_LITERAL(11, 165, 10), // "hasFeature"
QT_MOC_LITERAL(12, 176, 20), // "QDeviceInfo::Feature"
QT_MOC_LITERAL(13, 197, 7), // "feature"
QT_MOC_LITERAL(14, 205, 9), // "imeiCount"
QT_MOC_LITERAL(15, 215, 4), // "imei"
QT_MOC_LITERAL(16, 220, 15), // "interfaceNumber"
QT_MOC_LITERAL(17, 236, 12), // "manufacturer"
QT_MOC_LITERAL(18, 249, 5), // "model"
QT_MOC_LITERAL(19, 255, 11), // "productName"
QT_MOC_LITERAL(20, 267, 14), // "uniqueDeviceID"
QT_MOC_LITERAL(21, 282, 7), // "version"
QT_MOC_LITERAL(22, 290, 20), // "QDeviceInfo::Version"
QT_MOC_LITERAL(23, 311, 4), // "type"
QT_MOC_LITERAL(24, 316, 19), // "operatingSystemName"
QT_MOC_LITERAL(25, 336, 9), // "boardName"
QT_MOC_LITERAL(26, 346, 14), // "activatedLocks"
QT_MOC_LITERAL(27, 361, 13), // "LockTypeFlags"
QT_MOC_LITERAL(28, 375, 12), // "enabledLocks"
QT_MOC_LITERAL(29, 388, 12), // "thermalState"
QT_MOC_LITERAL(30, 401, 12), // "ThermalState"
QT_MOC_LITERAL(31, 414, 26), // "currentBluetoothPowerState"
QT_MOC_LITERAL(32, 441, 7), // "Feature"
QT_MOC_LITERAL(33, 449, 16), // "BluetoothFeature"
QT_MOC_LITERAL(34, 466, 13), // "CameraFeature"
QT_MOC_LITERAL(35, 480, 14), // "FmRadioFeature"
QT_MOC_LITERAL(36, 495, 20), // "FmTransmitterFeature"
QT_MOC_LITERAL(37, 516, 15), // "InfraredFeature"
QT_MOC_LITERAL(38, 532, 10), // "LedFeature"
QT_MOC_LITERAL(39, 543, 17), // "MemoryCardFeature"
QT_MOC_LITERAL(40, 561, 10), // "UsbFeature"
QT_MOC_LITERAL(41, 572, 16), // "VibrationFeature"
QT_MOC_LITERAL(42, 589, 11), // "WlanFeature"
QT_MOC_LITERAL(43, 601, 10), // "SimFeature"
QT_MOC_LITERAL(44, 612, 18), // "PositioningFeature"
QT_MOC_LITERAL(45, 631, 15), // "VideoOutFeature"
QT_MOC_LITERAL(46, 647, 14), // "HapticsFeature"
QT_MOC_LITERAL(47, 662, 10), // "NfcFeature"
QT_MOC_LITERAL(48, 673, 8), // "LockType"
QT_MOC_LITERAL(49, 682, 6), // "NoLock"
QT_MOC_LITERAL(50, 689, 7), // "PinLock"
QT_MOC_LITERAL(51, 697, 19), // "TouchOrKeyboardLock"
QT_MOC_LITERAL(52, 717, 11), // "UnknownLock"
QT_MOC_LITERAL(53, 729, 14), // "UnknownThermal"
QT_MOC_LITERAL(54, 744, 13), // "NormalThermal"
QT_MOC_LITERAL(55, 758, 14), // "WarningThermal"
QT_MOC_LITERAL(56, 773, 12), // "AlertThermal"
QT_MOC_LITERAL(57, 786, 12), // "ErrorThermal"
QT_MOC_LITERAL(58, 799, 7), // "Version"
QT_MOC_LITERAL(59, 807, 2), // "Os"
QT_MOC_LITERAL(60, 810, 8) // "Firmware"

    },
    "QDeviceInfo\0activatedLocksChanged\0\0"
    "QDeviceInfo::LockTypeFlags\0types\0"
    "enabledLocksChanged\0thermalStateChanged\0"
    "QDeviceInfo::ThermalState\0state\0"
    "bluetoothStateChanged\0on\0hasFeature\0"
    "QDeviceInfo::Feature\0feature\0imeiCount\0"
    "imei\0interfaceNumber\0manufacturer\0"
    "model\0productName\0uniqueDeviceID\0"
    "version\0QDeviceInfo::Version\0type\0"
    "operatingSystemName\0boardName\0"
    "activatedLocks\0LockTypeFlags\0enabledLocks\0"
    "thermalState\0ThermalState\0"
    "currentBluetoothPowerState\0Feature\0"
    "BluetoothFeature\0CameraFeature\0"
    "FmRadioFeature\0FmTransmitterFeature\0"
    "InfraredFeature\0LedFeature\0MemoryCardFeature\0"
    "UsbFeature\0VibrationFeature\0WlanFeature\0"
    "SimFeature\0PositioningFeature\0"
    "VideoOutFeature\0HapticsFeature\0"
    "NfcFeature\0LockType\0NoLock\0PinLock\0"
    "TouchOrKeyboardLock\0UnknownLock\0"
    "UnknownThermal\0NormalThermal\0"
    "WarningThermal\0AlertThermal\0ErrorThermal\0"
    "Version\0Os\0Firmware"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeviceInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       4,  112, // properties
       5,  128, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       5,    1,   87,    2, 0x06 /* Public */,
       6,    1,   90,    2, 0x06 /* Public */,
       9,    1,   93,    2, 0x06 /* Public */,

 // methods: name, argc, parameters, tag, flags
      11,    1,   96,    2, 0x02 /* Public */,
      14,    0,   99,    2, 0x02 /* Public */,
      15,    1,  100,    2, 0x02 /* Public */,
      17,    0,  103,    2, 0x02 /* Public */,
      18,    0,  104,    2, 0x02 /* Public */,
      19,    0,  105,    2, 0x02 /* Public */,
      20,    0,  106,    2, 0x02 /* Public */,
      21,    1,  107,    2, 0x02 /* Public */,
      24,    0,  110,    2, 0x02 /* Public */,
      25,    0,  111,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Bool,   10,

 // methods: parameters
    QMetaType::Bool, 0x80000000 | 12,   13,
    QMetaType::Int,
    QMetaType::QString, QMetaType::Int,   16,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString,
    QMetaType::QString, 0x80000000 | 22,   23,
    QMetaType::QString,
    QMetaType::QString,

 // properties: name, type, flags
      26, 0x80000000 | 27, 0x00495009,
      28, 0x80000000 | 27, 0x00495009,
      29, 0x80000000 | 30, 0x00495009,
      31, QMetaType::Bool, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,

 // enums: name, alias, flags, count, data
      32,   32, 0x0,   15,  153,
      48,   48, 0x1,    4,  183,
      27,   48, 0x1,    4,  191,
      30,   30, 0x0,    5,  199,
      58,   58, 0x0,    2,  209,

 // enum data: key, value
      33, uint(QDeviceInfo::BluetoothFeature),
      34, uint(QDeviceInfo::CameraFeature),
      35, uint(QDeviceInfo::FmRadioFeature),
      36, uint(QDeviceInfo::FmTransmitterFeature),
      37, uint(QDeviceInfo::InfraredFeature),
      38, uint(QDeviceInfo::LedFeature),
      39, uint(QDeviceInfo::MemoryCardFeature),
      40, uint(QDeviceInfo::UsbFeature),
      41, uint(QDeviceInfo::VibrationFeature),
      42, uint(QDeviceInfo::WlanFeature),
      43, uint(QDeviceInfo::SimFeature),
      44, uint(QDeviceInfo::PositioningFeature),
      45, uint(QDeviceInfo::VideoOutFeature),
      46, uint(QDeviceInfo::HapticsFeature),
      47, uint(QDeviceInfo::NfcFeature),
      49, uint(QDeviceInfo::NoLock),
      50, uint(QDeviceInfo::PinLock),
      51, uint(QDeviceInfo::TouchOrKeyboardLock),
      52, uint(QDeviceInfo::UnknownLock),
      49, uint(QDeviceInfo::NoLock),
      50, uint(QDeviceInfo::PinLock),
      51, uint(QDeviceInfo::TouchOrKeyboardLock),
      52, uint(QDeviceInfo::UnknownLock),
      53, uint(QDeviceInfo::UnknownThermal),
      54, uint(QDeviceInfo::NormalThermal),
      55, uint(QDeviceInfo::WarningThermal),
      56, uint(QDeviceInfo::AlertThermal),
      57, uint(QDeviceInfo::ErrorThermal),
      59, uint(QDeviceInfo::Os),
      60, uint(QDeviceInfo::Firmware),

       0        // eod
};

void QDeviceInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeviceInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->activatedLocksChanged((*reinterpret_cast< QDeviceInfo::LockTypeFlags(*)>(_a[1]))); break;
        case 1: _t->enabledLocksChanged((*reinterpret_cast< QDeviceInfo::LockTypeFlags(*)>(_a[1]))); break;
        case 2: _t->thermalStateChanged((*reinterpret_cast< QDeviceInfo::ThermalState(*)>(_a[1]))); break;
        case 3: _t->bluetoothStateChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: { bool _r = _t->hasFeature((*reinterpret_cast< QDeviceInfo::Feature(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 5: { int _r = _t->imeiCount();
            if (_a[0]) *reinterpret_cast< int*>(_a[0]) = std::move(_r); }  break;
        case 6: { QString _r = _t->imei((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 7: { QString _r = _t->manufacturer();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 8: { QString _r = _t->model();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->productName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QString _r = _t->uniqueDeviceID();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 11: { QString _r = _t->version((*reinterpret_cast< QDeviceInfo::Version(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 12: { QString _r = _t->operatingSystemName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 13: { QString _r = _t->boardName();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeviceInfo::*)(QDeviceInfo::LockTypeFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeviceInfo::activatedLocksChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeviceInfo::*)(QDeviceInfo::LockTypeFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeviceInfo::enabledLocksChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeviceInfo::*)(QDeviceInfo::ThermalState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeviceInfo::thermalStateChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeviceInfo::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeviceInfo::bluetoothStateChanged)) {
                *result = 3;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeviceInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = QFlag(_t->activatedLocks()); break;
        case 1: *reinterpret_cast<int*>(_v) = QFlag(_t->enabledLocks()); break;
        case 2: *reinterpret_cast< ThermalState*>(_v) = _t->thermalState(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->currentBluetoothPowerState(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeviceInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeviceInfo.data,
    qt_meta_data_QDeviceInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeviceInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeviceInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeviceInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeviceInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 4;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 4;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeviceInfo::activatedLocksChanged(QDeviceInfo::LockTypeFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QDeviceInfo::enabledLocksChanged(QDeviceInfo::LockTypeFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QDeviceInfo::thermalStateChanged(QDeviceInfo::ThermalState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeviceInfo::bluetoothStateChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
