/****************************************************************************
** Meta object code from reading C++ file 'qinputinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/systeminfo/qinputinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qinputinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QInputDevice_t {
    QByteArrayData data[13];
    char stringdata0[146];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QInputDevice_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QInputDevice_t qt_meta_stringdata_QInputDevice = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QInputDevice"
QT_MOC_LITERAL(1, 13, 10), // "properties"
QT_MOC_LITERAL(2, 24, 5), // "types"
QT_MOC_LITERAL(3, 30, 28), // "QInputDevice::InputTypeFlags"
QT_MOC_LITERAL(4, 59, 9), // "InputType"
QT_MOC_LITERAL(5, 69, 11), // "UnknownType"
QT_MOC_LITERAL(6, 81, 6), // "Button"
QT_MOC_LITERAL(7, 88, 5), // "Mouse"
QT_MOC_LITERAL(8, 94, 8), // "TouchPad"
QT_MOC_LITERAL(9, 103, 11), // "TouchScreen"
QT_MOC_LITERAL(10, 115, 8), // "Keyboard"
QT_MOC_LITERAL(11, 124, 6), // "Switch"
QT_MOC_LITERAL(12, 131, 14) // "InputTypeFlags"

    },
    "QInputDevice\0properties\0types\0"
    "QInputDevice::InputTypeFlags\0InputType\0"
    "UnknownType\0Button\0Mouse\0TouchPad\0"
    "TouchScreen\0Keyboard\0Switch\0InputTypeFlags"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QInputDevice[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       2,   14, // properties
       2,   20, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QVariantMap, 0x00095401,
       2, 0x80000000 | 3, 0x00095409,

 // enums: name, alias, flags, count, data
       4,    4, 0x0,    7,   30,
      12,    4, 0x1,    7,   44,

 // enum data: key, value
       5, uint(QInputDevice::UnknownType),
       6, uint(QInputDevice::Button),
       7, uint(QInputDevice::Mouse),
       8, uint(QInputDevice::TouchPad),
       9, uint(QInputDevice::TouchScreen),
      10, uint(QInputDevice::Keyboard),
      11, uint(QInputDevice::Switch),
       5, uint(QInputDevice::UnknownType),
       6, uint(QInputDevice::Button),
       7, uint(QInputDevice::Mouse),
       8, uint(QInputDevice::TouchPad),
       9, uint(QInputDevice::TouchScreen),
      10, uint(QInputDevice::Keyboard),
      11, uint(QInputDevice::Switch),

       0        // eod
};

void QInputDevice::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QInputDevice *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QVariantMap*>(_v) = _t->properties(); break;
        case 1: *reinterpret_cast< QInputDevice::InputTypeFlags*>(_v) = _t->types(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QInputDevice::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QInputDevice.data,
    qt_meta_data_QInputDevice,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QInputDevice::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QInputDevice::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QInputDevice.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QInputDevice::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 2;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 2;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QInputInfoManager_t {
    QByteArrayData data[19];
    char stringdata0[227];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QInputInfoManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QInputInfoManager_t qt_meta_stringdata_QInputInfoManager = {
    {
QT_MOC_LITERAL(0, 0, 17), // "QInputInfoManager"
QT_MOC_LITERAL(1, 18, 5), // "ready"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 11), // "deviceAdded"
QT_MOC_LITERAL(4, 37, 13), // "QInputDevice*"
QT_MOC_LITERAL(5, 51, 11), // "inputDevice"
QT_MOC_LITERAL(6, 63, 13), // "deviceRemoved"
QT_MOC_LITERAL(7, 77, 8), // "deviceId"
QT_MOC_LITERAL(8, 86, 12), // "countChanged"
QT_MOC_LITERAL(9, 99, 5), // "count"
QT_MOC_LITERAL(10, 105, 13), // "filterChanged"
QT_MOC_LITERAL(11, 119, 28), // "QInputDevice::InputTypeFlags"
QT_MOC_LITERAL(12, 148, 11), // "filterFlags"
QT_MOC_LITERAL(13, 160, 11), // "addedDevice"
QT_MOC_LITERAL(14, 172, 10), // "devicePath"
QT_MOC_LITERAL(15, 183, 13), // "removedDevice"
QT_MOC_LITERAL(16, 197, 12), // "privateReady"
QT_MOC_LITERAL(17, 210, 6), // "filter"
QT_MOC_LITERAL(18, 217, 9) // "lastAdded"

    },
    "QInputInfoManager\0ready\0\0deviceAdded\0"
    "QInputDevice*\0inputDevice\0deviceRemoved\0"
    "deviceId\0countChanged\0count\0filterChanged\0"
    "QInputDevice::InputTypeFlags\0filterFlags\0"
    "addedDevice\0devicePath\0removedDevice\0"
    "privateReady\0filter\0lastAdded"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QInputInfoManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       3,   74, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    1,   55,    2, 0x06 /* Public */,
       6,    1,   58,    2, 0x06 /* Public */,
       8,    1,   61,    2, 0x06 /* Public */,
      10,    1,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    1,   67,    2, 0x08 /* Private */,
      15,    1,   70,    2, 0x08 /* Private */,
      16,    0,   73,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, 0x80000000 | 11,   12,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,   14,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void,

 // properties: name, type, flags
       9, QMetaType::Int, 0x00495001,
      17, 0x80000000 | 11, 0x0049510b,
      18, 0x80000000 | 4, 0x00495009,

 // properties: notify_signal_id
       3,
       4,
       1,

       0        // eod
};

void QInputInfoManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QInputInfoManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->ready(); break;
        case 1: _t->deviceAdded((*reinterpret_cast< QInputDevice*(*)>(_a[1]))); break;
        case 2: _t->deviceRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->countChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->filterChanged((*reinterpret_cast< QInputDevice::InputTypeFlags(*)>(_a[1]))); break;
        case 5: _t->addedDevice((*reinterpret_cast< QInputDevice*(*)>(_a[1]))); break;
        case 6: _t->removedDevice((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->privateReady(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInputDevice* >(); break;
            }
            break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInputDevice* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QInputInfoManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QInputInfoManager::ready)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QInputInfoManager::*)(QInputDevice * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QInputInfoManager::deviceAdded)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QInputInfoManager::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QInputInfoManager::deviceRemoved)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QInputInfoManager::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QInputInfoManager::countChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QInputInfoManager::*)(QInputDevice::InputTypeFlags );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QInputInfoManager::filterChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QInputDevice* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QInputInfoManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->count(); break;
        case 1: *reinterpret_cast< QInputDevice::InputTypeFlags*>(_v) = _t->filter(); break;
        case 2: *reinterpret_cast< QInputDevice**>(_v) = _t->lastAdded(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QInputInfoManager *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setFilter(*reinterpret_cast< QInputDevice::InputTypeFlags*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

static const QMetaObject::SuperData qt_meta_extradata_QInputInfoManager[] = {
    QMetaObject::SuperData::link<QInputDevice::staticMetaObject>(),
    nullptr
};

QT_INIT_METAOBJECT const QMetaObject QInputInfoManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QInputInfoManager.data,
    qt_meta_data_QInputInfoManager,
    qt_static_metacall,
    qt_meta_extradata_QInputInfoManager,
    nullptr
} };


const QMetaObject *QInputInfoManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QInputInfoManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QInputInfoManager.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QInputInfoManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 3;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 3;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QInputInfoManager::ready()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QInputInfoManager::deviceAdded(QInputDevice * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QInputInfoManager::deviceRemoved(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QInputInfoManager::countChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QInputInfoManager::filterChanged(QInputDevice::InputTypeFlags _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
