/****************************************************************************
** Meta object code from reading C++ file 'qnetworkinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/systeminfo/qnetworkinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qnetworkinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QNetworkInfo_t {
    QByteArrayData data[26];
    char stringdata0[428];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QNetworkInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QNetworkInfo_t qt_meta_stringdata_QNetworkInfo = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QNetworkInfo"
QT_MOC_LITERAL(1, 13, 13), // "cellIdChanged"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 15), // "interfaceDevice"
QT_MOC_LITERAL(4, 44, 2), // "id"
QT_MOC_LITERAL(5, 47, 32), // "currentCellDataTechnologyChanged"
QT_MOC_LITERAL(6, 80, 32), // "QNetworkInfo::CellDataTechnology"
QT_MOC_LITERAL(7, 113, 4), // "tech"
QT_MOC_LITERAL(8, 118, 31), // "currentMobileCountryCodeChanged"
QT_MOC_LITERAL(9, 150, 3), // "mcc"
QT_MOC_LITERAL(10, 154, 31), // "currentMobileNetworkCodeChanged"
QT_MOC_LITERAL(11, 186, 3), // "mnc"
QT_MOC_LITERAL(12, 190, 25), // "currentNetworkModeChanged"
QT_MOC_LITERAL(13, 216, 25), // "QNetworkInfo::NetworkMode"
QT_MOC_LITERAL(14, 242, 4), // "mode"
QT_MOC_LITERAL(15, 247, 23), // "locationAreaCodeChanged"
QT_MOC_LITERAL(16, 271, 3), // "lac"
QT_MOC_LITERAL(17, 275, 28), // "networkInterfaceCountChanged"
QT_MOC_LITERAL(18, 304, 5), // "count"
QT_MOC_LITERAL(19, 310, 18), // "networkNameChanged"
QT_MOC_LITERAL(20, 329, 4), // "name"
QT_MOC_LITERAL(21, 334, 28), // "networkSignalStrengthChanged"
QT_MOC_LITERAL(22, 363, 8), // "strength"
QT_MOC_LITERAL(23, 372, 20), // "networkStatusChanged"
QT_MOC_LITERAL(24, 393, 27), // "QNetworkInfo::NetworkStatus"
QT_MOC_LITERAL(25, 421, 6) // "status"

    },
    "QNetworkInfo\0cellIdChanged\0\0interfaceDevice\0"
    "id\0currentCellDataTechnologyChanged\0"
    "QNetworkInfo::CellDataTechnology\0tech\0"
    "currentMobileCountryCodeChanged\0mcc\0"
    "currentMobileNetworkCodeChanged\0mnc\0"
    "currentNetworkModeChanged\0"
    "QNetworkInfo::NetworkMode\0mode\0"
    "locationAreaCodeChanged\0lac\0"
    "networkInterfaceCountChanged\0count\0"
    "networkNameChanged\0name\0"
    "networkSignalStrengthChanged\0strength\0"
    "networkStatusChanged\0QNetworkInfo::NetworkStatus\0"
    "status"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QNetworkInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   64,    2, 0x06 /* Public */,
       5,    2,   69,    2, 0x06 /* Public */,
       8,    2,   74,    2, 0x06 /* Public */,
      10,    2,   79,    2, 0x06 /* Public */,
      12,    1,   84,    2, 0x06 /* Public */,
      15,    2,   87,    2, 0x06 /* Public */,
      17,    2,   92,    2, 0x06 /* Public */,
      19,    3,   97,    2, 0x06 /* Public */,
      21,    3,  104,    2, 0x06 /* Public */,
      23,    3,  111,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    3,    4,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 6,    3,    7,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    3,    9,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    3,   11,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,    3,   16,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Int,   14,   18,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Int, QMetaType::QString,   14,    3,   20,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Int, QMetaType::Int,   14,    3,   22,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Int, 0x80000000 | 24,   14,    3,   25,

       0        // eod
};

void QNetworkInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QNetworkInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->cellIdChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->currentCellDataTechnologyChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QNetworkInfo::CellDataTechnology(*)>(_a[2]))); break;
        case 2: _t->currentMobileCountryCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->currentMobileNetworkCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 4: _t->currentNetworkModeChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1]))); break;
        case 5: _t->locationAreaCodeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 6: _t->networkInterfaceCountChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->networkNameChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3]))); break;
        case 8: _t->networkSignalStrengthChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 9: _t->networkStatusChanged((*reinterpret_cast< QNetworkInfo::NetworkMode(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< QNetworkInfo::NetworkStatus(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::cellIdChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(int , QNetworkInfo::CellDataTechnology );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::currentCellDataTechnologyChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::currentMobileCountryCodeChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::currentMobileNetworkCodeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(QNetworkInfo::NetworkMode );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::currentNetworkModeChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::locationAreaCodeChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(QNetworkInfo::NetworkMode , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::networkInterfaceCountChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(QNetworkInfo::NetworkMode , int , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::networkNameChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(QNetworkInfo::NetworkMode , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::networkSignalStrengthChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QNetworkInfo::*)(QNetworkInfo::NetworkMode , int , QNetworkInfo::NetworkStatus );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QNetworkInfo::networkStatusChanged)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject QNetworkInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QNetworkInfo.data,
    qt_meta_data_QNetworkInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QNetworkInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QNetworkInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QNetworkInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QNetworkInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void QNetworkInfo::cellIdChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QNetworkInfo::currentCellDataTechnologyChanged(int _t1, QNetworkInfo::CellDataTechnology _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QNetworkInfo::currentMobileCountryCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QNetworkInfo::currentMobileNetworkCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QNetworkInfo::currentNetworkModeChanged(QNetworkInfo::NetworkMode _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QNetworkInfo::locationAreaCodeChanged(int _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QNetworkInfo::networkInterfaceCountChanged(QNetworkInfo::NetworkMode _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QNetworkInfo::networkNameChanged(QNetworkInfo::NetworkMode _t1, int _t2, const QString & _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QNetworkInfo::networkSignalStrengthChanged(QNetworkInfo::NetworkMode _t1, int _t2, int _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QNetworkInfo::networkStatusChanged(QNetworkInfo::NetworkMode _t1, int _t2, QNetworkInfo::NetworkStatus _t3)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
