/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeserviceold_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/serviceframework/qdeclarativeserviceold_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeserviceold_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeService_t {
    QByteArrayData data[18];
    char stringdata0[266];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeService_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeService_t qt_meta_stringdata_QDeclarativeService = {
    {
QT_MOC_LITERAL(0, 0, 19), // "QDeclarativeService"
QT_MOC_LITERAL(1, 20, 12), // "validChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 20), // "serviceObjectChanged"
QT_MOC_LITERAL(4, 55, 20), // "interfaceNameChanged"
QT_MOC_LITERAL(5, 76, 18), // "serviceNameChanged"
QT_MOC_LITERAL(6, 95, 19), // "majorVersionChanged"
QT_MOC_LITERAL(7, 115, 19), // "minorVersionChanged"
QT_MOC_LITERAL(8, 135, 5), // "error"
QT_MOC_LITERAL(9, 141, 11), // "errorString"
QT_MOC_LITERAL(10, 153, 8), // "IPCFault"
QT_MOC_LITERAL(11, 162, 31), // "QService::UnrecoverableIPCError"
QT_MOC_LITERAL(12, 194, 13), // "interfaceName"
QT_MOC_LITERAL(13, 208, 11), // "serviceName"
QT_MOC_LITERAL(14, 220, 12), // "majorVersion"
QT_MOC_LITERAL(15, 233, 12), // "minorVersion"
QT_MOC_LITERAL(16, 246, 5), // "valid"
QT_MOC_LITERAL(17, 252, 13) // "serviceObject"

    },
    "QDeclarativeService\0validChanged\0\0"
    "serviceObjectChanged\0interfaceNameChanged\0"
    "serviceNameChanged\0majorVersionChanged\0"
    "minorVersionChanged\0error\0errorString\0"
    "IPCFault\0QService::UnrecoverableIPCError\0"
    "interfaceName\0serviceName\0majorVersion\0"
    "minorVersion\0valid\0serviceObject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeService[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       8,   14, // methods
       7,   66, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   54,    2, 0x06 /* Public */,
       3,    0,   55,    2, 0x06 /* Public */,
       4,    0,   56,    2, 0x06 /* Public */,
       5,    0,   57,    2, 0x06 /* Public */,
       6,    0,   58,    2, 0x06 /* Public */,
       7,    0,   59,    2, 0x06 /* Public */,
       8,    1,   60,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   63,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 11,    2,

 // properties: name, type, flags
      12, QMetaType::QString, 0x00495103,
      13, QMetaType::QString, 0x00495103,
      14, QMetaType::Int, 0x00495103,
      15, QMetaType::Int, 0x00495103,
      16, QMetaType::Bool, 0x00495001,
      17, QMetaType::QObjectStar, 0x00495001,
       8, QMetaType::QString, 0x00495001,

 // properties: notify_signal_id
       2,
       3,
       4,
       5,
       0,
       1,
       6,

       0        // eod
};

void QDeclarativeService::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeService *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->validChanged(); break;
        case 1: _t->serviceObjectChanged(); break;
        case 2: _t->interfaceNameChanged(); break;
        case 3: _t->serviceNameChanged(); break;
        case 4: _t->majorVersionChanged(); break;
        case 5: _t->minorVersionChanged(); break;
        case 6: _t->error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->IPCFault((*reinterpret_cast< QService::UnrecoverableIPCError(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::validChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::serviceObjectChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::interfaceNameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::serviceNameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::majorVersionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::minorVersionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeService::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeService::error)) {
                *result = 6;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->interfaceName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->serviceName(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isValid(); break;
        case 5: *reinterpret_cast< QObject**>(_v) = _t->serviceObject(); break;
        case 6: *reinterpret_cast< QString*>(_v) = _t->lastError(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeService *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInterfaceName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setServiceName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setMajorVersion(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setMinorVersion(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeService::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeService.data,
    qt_meta_data_QDeclarativeService,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeService::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeService::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeService.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeService::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 8;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeService::validChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeService::serviceObjectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void QDeclarativeService::interfaceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeService::serviceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeService::majorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeService::minorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeService::error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}
struct qt_meta_stringdata_QDeclarativeServiceList_t {
    QByteArrayData data[24];
    char stringdata0[414];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeServiceList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeServiceList_t qt_meta_stringdata_QDeclarativeServiceList = {
    {
QT_MOC_LITERAL(0, 0, 23), // "QDeclarativeServiceList"
QT_MOC_LITERAL(1, 24, 14), // "resultsChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 15), // "servicesChanged"
QT_MOC_LITERAL(4, 56, 37), // "QQmlListProperty<QDeclarative..."
QT_MOC_LITERAL(5, 94, 18), // "serviceNameChanged"
QT_MOC_LITERAL(6, 113, 20), // "interfaceNameChanged"
QT_MOC_LITERAL(7, 134, 19), // "minorVersionChanged"
QT_MOC_LITERAL(8, 154, 19), // "majorVersionChanged"
QT_MOC_LITERAL(9, 174, 19), // "versionMatchChanged"
QT_MOC_LITERAL(10, 194, 34), // "monitorServiceRegistrationsCh..."
QT_MOC_LITERAL(11, 229, 20), // "servicesAddedRemoved"
QT_MOC_LITERAL(12, 250, 19), // "updateFilterResults"
QT_MOC_LITERAL(13, 270, 17), // "updateServiceList"
QT_MOC_LITERAL(14, 288, 11), // "serviceName"
QT_MOC_LITERAL(15, 300, 13), // "interfaceName"
QT_MOC_LITERAL(16, 314, 12), // "majorVersion"
QT_MOC_LITERAL(17, 327, 12), // "minorVersion"
QT_MOC_LITERAL(18, 340, 27), // "monitorServiceRegistrations"
QT_MOC_LITERAL(19, 368, 8), // "services"
QT_MOC_LITERAL(20, 377, 12), // "versionMatch"
QT_MOC_LITERAL(21, 390, 9), // "MatchRule"
QT_MOC_LITERAL(22, 400, 7), // "Minimum"
QT_MOC_LITERAL(23, 408, 5) // "Exact"

    },
    "QDeclarativeServiceList\0resultsChanged\0"
    "\0servicesChanged\0QQmlListProperty<QDeclarativeService>\0"
    "serviceNameChanged\0interfaceNameChanged\0"
    "minorVersionChanged\0majorVersionChanged\0"
    "versionMatchChanged\0"
    "monitorServiceRegistrationsChanged\0"
    "servicesAddedRemoved\0updateFilterResults\0"
    "updateServiceList\0serviceName\0"
    "interfaceName\0majorVersion\0minorVersion\0"
    "monitorServiceRegistrations\0services\0"
    "versionMatch\0MatchRule\0Minimum\0Exact"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeServiceList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       7,   82, // properties
       1,  110, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    1,   70,    2, 0x06 /* Public */,
       5,    0,   73,    2, 0x06 /* Public */,
       6,    0,   74,    2, 0x06 /* Public */,
       7,    0,   75,    2, 0x06 /* Public */,
       8,    0,   76,    2, 0x06 /* Public */,
       9,    0,   77,    2, 0x06 /* Public */,
      10,    0,   78,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,   79,    2, 0x09 /* Protected */,
      12,    0,   80,    2, 0x09 /* Protected */,
      13,    0,   81,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      14, QMetaType::QString, 0x00495103,
      15, QMetaType::QString, 0x00495103,
      16, QMetaType::Int, 0x00495103,
      17, QMetaType::Int, 0x00495103,
      18, QMetaType::Bool, 0x00495103,
      19, 0x80000000 | 4, 0x00495009,
      20, 0x80000000 | 21, 0x0049510b,

 // properties: notify_signal_id
       2,
       3,
       5,
       4,
       7,
       0,
       6,

 // enums: name, alias, flags, count, data
      21,   21, 0x0,    2,  115,

 // enum data: key, value
      22, uint(QDeclarativeServiceList::Minimum),
      23, uint(QDeclarativeServiceList::Exact),

       0        // eod
};

void QDeclarativeServiceList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeServiceList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->resultsChanged(); break;
        case 1: _t->servicesChanged((*reinterpret_cast< const QQmlListProperty<QDeclarativeService>(*)>(_a[1]))); break;
        case 2: _t->serviceNameChanged(); break;
        case 3: _t->interfaceNameChanged(); break;
        case 4: _t->minorVersionChanged(); break;
        case 5: _t->majorVersionChanged(); break;
        case 6: _t->versionMatchChanged(); break;
        case 7: _t->monitorServiceRegistrationsChanged(); break;
        case 8: _t->servicesAddedRemoved(); break;
        case 9: _t->updateFilterResults(); break;
        case 10: _t->updateServiceList(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativeService> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::resultsChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)(const QQmlListProperty<QDeclarativeService> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::servicesChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::serviceNameChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::interfaceNameChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::minorVersionChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::majorVersionChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::versionMatchChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceList::monitorServiceRegistrationsChanged)) {
                *result = 7;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativeService> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeServiceList *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->serviceName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->interfaceName(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->monitorServiceRegistrations(); break;
        case 5: *reinterpret_cast< QQmlListProperty<QDeclarativeService>*>(_v) = _t->services(); break;
        case 6: *reinterpret_cast< MatchRule*>(_v) = _t->versionMatch(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeServiceList *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServiceName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setInterfaceName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setMajorVersion(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setMinorVersion(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setMonitorServiceRegistrations(*reinterpret_cast< bool*>(_v)); break;
        case 6: _t->setVersionMatch(*reinterpret_cast< MatchRule*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeServiceList::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeServiceList.data,
    qt_meta_data_QDeclarativeServiceList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeServiceList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeServiceList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeServiceList.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeServiceList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeServiceList::resultsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void QDeclarativeServiceList::servicesChanged(const QQmlListProperty<QDeclarativeService> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QDeclarativeServiceList::serviceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void QDeclarativeServiceList::interfaceNameChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void QDeclarativeServiceList::minorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void QDeclarativeServiceList::majorVersionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void QDeclarativeServiceList::versionMatchChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void QDeclarativeServiceList::monitorServiceRegistrationsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
