/****************************************************************************
** Meta object code from reading C++ file 'qservicepackage_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/serviceframework/ipc/qservicepackage_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qservicepackage_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QServicePackage_t {
    QByteArrayData data[9];
    char stringdata0[101];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QServicePackage_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QServicePackage_t qt_meta_stringdata_QServicePackage = {
    {
QT_MOC_LITERAL(0, 0, 15), // "QServicePackage"
QT_MOC_LITERAL(1, 16, 4), // "Type"
QT_MOC_LITERAL(2, 21, 14), // "ObjectCreation"
QT_MOC_LITERAL(3, 36, 10), // "MethodCall"
QT_MOC_LITERAL(4, 47, 12), // "PropertyCall"
QT_MOC_LITERAL(5, 60, 12), // "ResponseType"
QT_MOC_LITERAL(6, 73, 12), // "NotAResponse"
QT_MOC_LITERAL(7, 86, 7), // "Success"
QT_MOC_LITERAL(8, 94, 6) // "Failed"

    },
    "QServicePackage\0Type\0ObjectCreation\0"
    "MethodCall\0PropertyCall\0ResponseType\0"
    "NotAResponse\0Success\0Failed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QServicePackage[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       2,   14, // enums/sets
       0,    0, // constructors
       4,       // flags
       0,       // signalCount

 // enums: name, alias, flags, count, data
       1,    1, 0x0,    3,   24,
       5,    5, 0x0,    3,   30,

 // enum data: key, value
       2, uint(QServicePackage::ObjectCreation),
       3, uint(QServicePackage::MethodCall),
       4, uint(QServicePackage::PropertyCall),
       6, uint(QServicePackage::NotAResponse),
       7, uint(QServicePackage::Success),
       8, uint(QServicePackage::Failed),

       0        // eod
};

QT_INIT_METAOBJECT const QMetaObject QServicePackage::staticMetaObject = { {
    nullptr,
    qt_meta_stringdata_QServicePackage.data,
    qt_meta_data_QServicePackage,
    nullptr,
    nullptr,
    nullptr
} };

QT_WARNING_POP
QT_END_MOC_NAMESPACE
