/****************************************************************************
** Meta object code from reading C++ file 'qbatteryinfo.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../git/src/systeminfo/qbatteryinfo.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qbatteryinfo.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QBatteryInfo_t {
    QByteArrayData data[62];
    char stringdata0[858];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QBatteryInfo_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QBatteryInfo_t qt_meta_stringdata_QBatteryInfo = {
    {
QT_MOC_LITERAL(0, 0, 12), // "QBatteryInfo"
QT_MOC_LITERAL(1, 13, 19), // "batteryIndexChanged"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 12), // "batteryIndex"
QT_MOC_LITERAL(4, 47, 12), // "validChanged"
QT_MOC_LITERAL(5, 60, 7), // "isValid"
QT_MOC_LITERAL(6, 68, 19), // "batteryCountChanged"
QT_MOC_LITERAL(7, 88, 5), // "count"
QT_MOC_LITERAL(8, 94, 18), // "chargerTypeChanged"
QT_MOC_LITERAL(9, 113, 25), // "QBatteryInfo::ChargerType"
QT_MOC_LITERAL(10, 139, 4), // "type"
QT_MOC_LITERAL(11, 144, 20), // "chargingStateChanged"
QT_MOC_LITERAL(12, 165, 27), // "QBatteryInfo::ChargingState"
QT_MOC_LITERAL(13, 193, 5), // "state"
QT_MOC_LITERAL(14, 199, 12), // "levelChanged"
QT_MOC_LITERAL(15, 212, 5), // "level"
QT_MOC_LITERAL(16, 218, 18), // "currentFlowChanged"
QT_MOC_LITERAL(17, 237, 4), // "flow"
QT_MOC_LITERAL(18, 242, 17), // "cycleCountChanged"
QT_MOC_LITERAL(19, 260, 10), // "cycleCount"
QT_MOC_LITERAL(20, 271, 24), // "remainingCapacityChanged"
QT_MOC_LITERAL(21, 296, 8), // "capacity"
QT_MOC_LITERAL(22, 305, 28), // "remainingChargingTimeChanged"
QT_MOC_LITERAL(23, 334, 7), // "seconds"
QT_MOC_LITERAL(24, 342, 14), // "voltageChanged"
QT_MOC_LITERAL(25, 357, 7), // "voltage"
QT_MOC_LITERAL(26, 365, 18), // "levelStatusChanged"
QT_MOC_LITERAL(27, 384, 25), // "QBatteryInfo::LevelStatus"
QT_MOC_LITERAL(28, 410, 11), // "levelStatus"
QT_MOC_LITERAL(29, 422, 13), // "healthChanged"
QT_MOC_LITERAL(30, 436, 20), // "QBatteryInfo::Health"
QT_MOC_LITERAL(31, 457, 6), // "health"
QT_MOC_LITERAL(32, 464, 18), // "temperatureChanged"
QT_MOC_LITERAL(33, 483, 11), // "temperature"
QT_MOC_LITERAL(34, 495, 12), // "batteryCount"
QT_MOC_LITERAL(35, 508, 5), // "valid"
QT_MOC_LITERAL(36, 514, 11), // "currentFlow"
QT_MOC_LITERAL(37, 526, 15), // "maximumCapacity"
QT_MOC_LITERAL(38, 542, 17), // "remainingCapacity"
QT_MOC_LITERAL(39, 560, 21), // "remainingChargingTime"
QT_MOC_LITERAL(40, 582, 13), // "chargingState"
QT_MOC_LITERAL(41, 596, 13), // "ChargingState"
QT_MOC_LITERAL(42, 610, 11), // "chargerType"
QT_MOC_LITERAL(43, 622, 11), // "ChargerType"
QT_MOC_LITERAL(44, 634, 11), // "LevelStatus"
QT_MOC_LITERAL(45, 646, 6), // "Health"
QT_MOC_LITERAL(46, 653, 14), // "UnknownCharger"
QT_MOC_LITERAL(47, 668, 11), // "WallCharger"
QT_MOC_LITERAL(48, 680, 10), // "USBCharger"
QT_MOC_LITERAL(49, 691, 22), // "VariableCurrentCharger"
QT_MOC_LITERAL(50, 714, 20), // "UnknownChargingState"
QT_MOC_LITERAL(51, 735, 8), // "Charging"
QT_MOC_LITERAL(52, 744, 17), // "IdleChargingState"
QT_MOC_LITERAL(53, 762, 11), // "Discharging"
QT_MOC_LITERAL(54, 774, 12), // "LevelUnknown"
QT_MOC_LITERAL(55, 787, 10), // "LevelEmpty"
QT_MOC_LITERAL(56, 798, 8), // "LevelLow"
QT_MOC_LITERAL(57, 807, 7), // "LevelOk"
QT_MOC_LITERAL(58, 815, 9), // "LevelFull"
QT_MOC_LITERAL(59, 825, 13), // "HealthUnknown"
QT_MOC_LITERAL(60, 839, 8), // "HealthOk"
QT_MOC_LITERAL(61, 848, 9) // "HealthBad"

    },
    "QBatteryInfo\0batteryIndexChanged\0\0"
    "batteryIndex\0validChanged\0isValid\0"
    "batteryCountChanged\0count\0chargerTypeChanged\0"
    "QBatteryInfo::ChargerType\0type\0"
    "chargingStateChanged\0QBatteryInfo::ChargingState\0"
    "state\0levelChanged\0level\0currentFlowChanged\0"
    "flow\0cycleCountChanged\0cycleCount\0"
    "remainingCapacityChanged\0capacity\0"
    "remainingChargingTimeChanged\0seconds\0"
    "voltageChanged\0voltage\0levelStatusChanged\0"
    "QBatteryInfo::LevelStatus\0levelStatus\0"
    "healthChanged\0QBatteryInfo::Health\0"
    "health\0temperatureChanged\0temperature\0"
    "batteryCount\0valid\0currentFlow\0"
    "maximumCapacity\0remainingCapacity\0"
    "remainingChargingTime\0chargingState\0"
    "ChargingState\0chargerType\0ChargerType\0"
    "LevelStatus\0Health\0UnknownCharger\0"
    "WallCharger\0USBCharger\0VariableCurrentCharger\0"
    "UnknownChargingState\0Charging\0"
    "IdleChargingState\0Discharging\0"
    "LevelUnknown\0LevelEmpty\0LevelLow\0"
    "LevelOk\0LevelFull\0HealthUnknown\0"
    "HealthOk\0HealthBad"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QBatteryInfo[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
      15,  126, // properties
       4,  186, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       4,    1,   87,    2, 0x06 /* Public */,
       6,    1,   90,    2, 0x06 /* Public */,
       8,    1,   93,    2, 0x06 /* Public */,
      11,    1,   96,    2, 0x06 /* Public */,
      14,    1,   99,    2, 0x06 /* Public */,
      16,    1,  102,    2, 0x06 /* Public */,
      18,    1,  105,    2, 0x06 /* Public */,
      20,    1,  108,    2, 0x06 /* Public */,
      22,    1,  111,    2, 0x06 /* Public */,
      24,    1,  114,    2, 0x06 /* Public */,
      26,    1,  117,    2, 0x06 /* Public */,
      29,    1,  120,    2, 0x06 /* Public */,
      32,    1,  123,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Bool,    5,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, 0x80000000 | 27,   28,
    QMetaType::Void, 0x80000000 | 30,   31,
    QMetaType::Void, QMetaType::Float,   33,

 // properties: name, type, flags
      34, QMetaType::Int, 0x00495001,
       3, QMetaType::Int, 0x00495103,
      35, QMetaType::Bool, 0x00495001,
      15, QMetaType::Int, 0x00495001,
      36, QMetaType::Int, 0x00495001,
      19, QMetaType::Int, 0x00495001,
      37, QMetaType::Int, 0x00095001,
      38, QMetaType::Int, 0x00495001,
      39, QMetaType::Int, 0x00495001,
      25, QMetaType::Int, 0x00495001,
      40, 0x80000000 | 41, 0x00495009,
      42, 0x80000000 | 43, 0x00495009,
      28, 0x80000000 | 44, 0x00495009,
      31, 0x80000000 | 45, 0x00495009,
      33, QMetaType::Float, 0x00495001,

 // properties: notify_signal_id
       2,
       0,
       1,
       5,
       6,
       7,
       0,
       8,
       9,
      10,
       4,
       3,
      11,
      12,
      13,

 // enums: name, alias, flags, count, data
      43,   43, 0x0,    4,  206,
      41,   41, 0x0,    4,  214,
      44,   44, 0x0,    5,  222,
      45,   45, 0x0,    3,  232,

 // enum data: key, value
      46, uint(QBatteryInfo::UnknownCharger),
      47, uint(QBatteryInfo::WallCharger),
      48, uint(QBatteryInfo::USBCharger),
      49, uint(QBatteryInfo::VariableCurrentCharger),
      50, uint(QBatteryInfo::UnknownChargingState),
      51, uint(QBatteryInfo::Charging),
      52, uint(QBatteryInfo::IdleChargingState),
      53, uint(QBatteryInfo::Discharging),
      54, uint(QBatteryInfo::LevelUnknown),
      55, uint(QBatteryInfo::LevelEmpty),
      56, uint(QBatteryInfo::LevelLow),
      57, uint(QBatteryInfo::LevelOk),
      58, uint(QBatteryInfo::LevelFull),
      59, uint(QBatteryInfo::HealthUnknown),
      60, uint(QBatteryInfo::HealthOk),
      61, uint(QBatteryInfo::HealthBad),

       0        // eod
};

void QBatteryInfo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QBatteryInfo *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->batteryIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->validChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 2: _t->batteryCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->chargerTypeChanged((*reinterpret_cast< QBatteryInfo::ChargerType(*)>(_a[1]))); break;
        case 4: _t->chargingStateChanged((*reinterpret_cast< QBatteryInfo::ChargingState(*)>(_a[1]))); break;
        case 5: _t->levelChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->currentFlowChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->cycleCountChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->remainingCapacityChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->remainingChargingTimeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->voltageChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->levelStatusChanged((*reinterpret_cast< QBatteryInfo::LevelStatus(*)>(_a[1]))); break;
        case 12: _t->healthChanged((*reinterpret_cast< QBatteryInfo::Health(*)>(_a[1]))); break;
        case 13: _t->temperatureChanged((*reinterpret_cast< float(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::batteryIndexChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::validChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::batteryCountChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(QBatteryInfo::ChargerType );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::chargerTypeChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(QBatteryInfo::ChargingState );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::chargingStateChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::levelChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::currentFlowChanged)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::cycleCountChanged)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::remainingCapacityChanged)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::remainingChargingTimeChanged)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::voltageChanged)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(QBatteryInfo::LevelStatus );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::levelStatusChanged)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(QBatteryInfo::Health );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::healthChanged)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (QBatteryInfo::*)(float );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QBatteryInfo::temperatureChanged)) {
                *result = 13;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QBatteryInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->batteryCount(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->batteryIndex(); break;
        case 2: *reinterpret_cast< bool*>(_v) = _t->isValid(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->level(); break;
        case 4: *reinterpret_cast< int*>(_v) = _t->currentFlow(); break;
        case 5: *reinterpret_cast< int*>(_v) = _t->cycleCount(); break;
        case 6: *reinterpret_cast< int*>(_v) = _t->maximumCapacity(); break;
        case 7: *reinterpret_cast< int*>(_v) = _t->remainingCapacity(); break;
        case 8: *reinterpret_cast< int*>(_v) = _t->remainingChargingTime(); break;
        case 9: *reinterpret_cast< int*>(_v) = _t->voltage(); break;
        case 10: *reinterpret_cast< ChargingState*>(_v) = _t->chargingState(); break;
        case 11: *reinterpret_cast< ChargerType*>(_v) = _t->chargerType(); break;
        case 12: *reinterpret_cast< LevelStatus*>(_v) = _t->levelStatus(); break;
        case 13: *reinterpret_cast< Health*>(_v) = _t->health(); break;
        case 14: *reinterpret_cast< float*>(_v) = _t->temperature(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QBatteryInfo *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 1: _t->setBatteryIndex(*reinterpret_cast< int*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QBatteryInfo::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QBatteryInfo.data,
    qt_meta_data_QBatteryInfo,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QBatteryInfo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QBatteryInfo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QBatteryInfo.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QBatteryInfo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 15;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 15;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QBatteryInfo::batteryIndexChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QBatteryInfo::validChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QBatteryInfo::batteryCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QBatteryInfo::chargerTypeChanged(QBatteryInfo::ChargerType _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QBatteryInfo::chargingStateChanged(QBatteryInfo::ChargingState _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QBatteryInfo::levelChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QBatteryInfo::currentFlowChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void QBatteryInfo::cycleCountChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void QBatteryInfo::remainingCapacityChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void QBatteryInfo::remainingChargingTimeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void QBatteryInfo::voltageChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void QBatteryInfo::levelStatusChanged(QBatteryInfo::LevelStatus _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void QBatteryInfo::healthChanged(QBatteryInfo::Health _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void QBatteryInfo::temperatureChanged(float _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
