/****************************************************************************
** Meta object code from reading C++ file 'qdeclarativeservice_p.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.4)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../git/src/imports/serviceframework/qdeclarativeservice_p.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qdeclarativeservice_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.4. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_QDeclarativeServiceDescriptor_t {
    QByteArrayData data[6];
    char stringdata0[88];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeServiceDescriptor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeServiceDescriptor_t qt_meta_stringdata_QDeclarativeServiceDescriptor = {
    {
QT_MOC_LITERAL(0, 0, 29), // "QDeclarativeServiceDescriptor"
QT_MOC_LITERAL(1, 30, 11), // "serviceName"
QT_MOC_LITERAL(2, 42, 13), // "interfaceName"
QT_MOC_LITERAL(3, 56, 12), // "majorVersion"
QT_MOC_LITERAL(4, 69, 12), // "minorVersion"
QT_MOC_LITERAL(5, 82, 5) // "valid"

    },
    "QDeclarativeServiceDescriptor\0serviceName\0"
    "interfaceName\0majorVersion\0minorVersion\0"
    "valid"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeServiceDescriptor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       5,   14, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // properties: name, type, flags
       1, QMetaType::QString, 0x00095401,
       2, QMetaType::QString, 0x00095401,
       3, QMetaType::Int, 0x00095401,
       4, QMetaType::Int, 0x00095401,
       5, QMetaType::Bool, 0x00095401,

       0        // eod
};

void QDeclarativeServiceDescriptor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{

#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeServiceDescriptor *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->serviceName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->interfaceName(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->isValid(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeServiceDescriptor::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeServiceDescriptor.data,
    qt_meta_data_QDeclarativeServiceDescriptor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeServiceDescriptor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeServiceDescriptor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeServiceDescriptor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QServiceInterfaceDescriptor"))
        return static_cast< QServiceInterfaceDescriptor*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeServiceDescriptor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    
#ifndef QT_NO_PROPERTIES
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}
struct qt_meta_stringdata_QDeclarativeServiceLoader_t {
    QByteArrayData data[29];
    char stringdata0[434];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeServiceLoader_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeServiceLoader_t qt_meta_stringdata_QDeclarativeServiceLoader = {
    {
QT_MOC_LITERAL(0, 0, 25), // "QDeclarativeServiceLoader"
QT_MOC_LITERAL(1, 26, 20), // "interfaceNameChanged"
QT_MOC_LITERAL(2, 47, 0), // ""
QT_MOC_LITERAL(3, 48, 3), // "arg"
QT_MOC_LITERAL(4, 52, 24), // "serviceDescriptorChanged"
QT_MOC_LITERAL(5, 77, 28), // "QServiceInterfaceDescriptor*"
QT_MOC_LITERAL(6, 106, 13), // "statusChanged"
QT_MOC_LITERAL(7, 120, 6), // "Status"
QT_MOC_LITERAL(8, 127, 19), // "asynchronousChanged"
QT_MOC_LITERAL(9, 147, 20), // "serviceObjectChanged"
QT_MOC_LITERAL(10, 168, 16), // "setInterfaceName"
QT_MOC_LITERAL(11, 185, 20), // "setServiceDescriptor"
QT_MOC_LITERAL(12, 206, 30), // "QDeclarativeServiceDescriptor*"
QT_MOC_LITERAL(13, 237, 15), // "setAsynchronous"
QT_MOC_LITERAL(14, 253, 9), // "setStatus"
QT_MOC_LITERAL(15, 263, 12), // "startLoading"
QT_MOC_LITERAL(16, 276, 13), // "finishLoading"
QT_MOC_LITERAL(17, 290, 8), // "IPCFault"
QT_MOC_LITERAL(18, 299, 31), // "QService::UnrecoverableIPCError"
QT_MOC_LITERAL(19, 331, 11), // "errorString"
QT_MOC_LITERAL(20, 343, 13), // "interfaceName"
QT_MOC_LITERAL(21, 357, 17), // "serviceDescriptor"
QT_MOC_LITERAL(22, 375, 6), // "status"
QT_MOC_LITERAL(23, 382, 12), // "asynchronous"
QT_MOC_LITERAL(24, 395, 13), // "serviceObject"
QT_MOC_LITERAL(25, 409, 4), // "Null"
QT_MOC_LITERAL(26, 414, 5), // "Ready"
QT_MOC_LITERAL(27, 420, 7), // "Loading"
QT_MOC_LITERAL(28, 428, 5) // "Error"

    },
    "QDeclarativeServiceLoader\0"
    "interfaceNameChanged\0\0arg\0"
    "serviceDescriptorChanged\0"
    "QServiceInterfaceDescriptor*\0statusChanged\0"
    "Status\0asynchronousChanged\0"
    "serviceObjectChanged\0setInterfaceName\0"
    "setServiceDescriptor\0"
    "QDeclarativeServiceDescriptor*\0"
    "setAsynchronous\0setStatus\0startLoading\0"
    "finishLoading\0IPCFault\0"
    "QService::UnrecoverableIPCError\0"
    "errorString\0interfaceName\0serviceDescriptor\0"
    "status\0asynchronous\0serviceObject\0"
    "Null\0Ready\0Loading\0Error"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeServiceLoader[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       5,  112, // properties
       1,  132, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    1,   82,    2, 0x06 /* Public */,
       6,    1,   85,    2, 0x06 /* Public */,
       8,    1,   88,    2, 0x06 /* Public */,
       9,    1,   91,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,   94,    2, 0x0a /* Public */,
      11,    1,   97,    2, 0x0a /* Public */,
      13,    1,  100,    2, 0x0a /* Public */,
      14,    1,  103,    2, 0x08 /* Private */,
      15,    0,  106,    2, 0x08 /* Private */,
      16,    0,  107,    2, 0x08 /* Private */,
      17,    1,  108,    2, 0x08 /* Private */,

 // methods: name, argc, parameters, tag, flags
      19,    0,  111,    2, 0x02 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 5,    3,
    QMetaType::Void, 0x80000000 | 7,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::QObjectStar,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, 0x80000000 | 12,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, 0x80000000 | 7,    3,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 18,    2,

 // methods: parameters
    QMetaType::QString,

 // properties: name, type, flags
      20, QMetaType::QString, 0x00495103,
      21, 0x80000000 | 12, 0x0049510f,
      22, 0x80000000 | 7, 0x00495009,
      23, QMetaType::Bool, 0x00495103,
      24, QMetaType::QObjectStar, 0x00495001,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,

 // enums: name, alias, flags, count, data
       7,    7, 0x0,    4,  137,

 // enum data: key, value
      25, uint(QDeclarativeServiceLoader::Null),
      26, uint(QDeclarativeServiceLoader::Ready),
      27, uint(QDeclarativeServiceLoader::Loading),
      28, uint(QDeclarativeServiceLoader::Error),

       0        // eod
};

void QDeclarativeServiceLoader::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeServiceLoader *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->interfaceNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->serviceDescriptorChanged((*reinterpret_cast< QServiceInterfaceDescriptor*(*)>(_a[1]))); break;
        case 2: _t->statusChanged((*reinterpret_cast< Status(*)>(_a[1]))); break;
        case 3: _t->asynchronousChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->serviceObjectChanged((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 5: _t->setInterfaceName((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 6: _t->setServiceDescriptor((*reinterpret_cast< QDeclarativeServiceDescriptor*(*)>(_a[1]))); break;
        case 7: _t->setAsynchronous((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->setStatus((*reinterpret_cast< Status(*)>(_a[1]))); break;
        case 9: _t->startLoading(); break;
        case 10: _t->finishLoading(); break;
        case 11: _t->IPCFault((*reinterpret_cast< QService::UnrecoverableIPCError(*)>(_a[1]))); break;
        case 12: { QString _r = _t->errorString();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeServiceDescriptor* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeServiceLoader::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceLoader::interfaceNameChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceLoader::*)(QServiceInterfaceDescriptor * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceLoader::serviceDescriptorChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceLoader::*)(Status );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceLoader::statusChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceLoader::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceLoader::asynchronousChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceLoader::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceLoader::serviceObjectChanged)) {
                *result = 4;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDeclarativeServiceDescriptor* >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeServiceLoader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->interfaceName(); break;
        case 1: *reinterpret_cast< QDeclarativeServiceDescriptor**>(_v) = _t->serviceDescriptor(); break;
        case 2: *reinterpret_cast< Status*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast< bool*>(_v) = _t->asynchronous(); break;
        case 4: *reinterpret_cast< QObject**>(_v) = _t->serviceObject(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeServiceLoader *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setInterfaceName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setServiceDescriptor(*reinterpret_cast< QDeclarativeServiceDescriptor**>(_v)); break;
        case 3: _t->setAsynchronous(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        QDeclarativeServiceLoader *_t = static_cast<QDeclarativeServiceLoader *>(_o);
        (void)_t;
        switch (_id) {
        case 1: _t->resetServiceDescriptor(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeServiceLoader::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeServiceLoader.data,
    qt_meta_data_QDeclarativeServiceLoader,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeServiceLoader::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeServiceLoader::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeServiceLoader.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeServiceLoader::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 5;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 5;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeServiceLoader::interfaceNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QDeclarativeServiceLoader::serviceDescriptorChanged(QServiceInterfaceDescriptor * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QDeclarativeServiceLoader::statusChanged(Status _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeclarativeServiceLoader::asynchronousChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QDeclarativeServiceLoader::serviceObjectChanged(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
struct qt_meta_stringdata_QDeclarativeServiceFilter_t {
    QByteArrayData data[27];
    char stringdata0[536];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_QDeclarativeServiceFilter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_QDeclarativeServiceFilter_t qt_meta_stringdata_QDeclarativeServiceFilter = {
    {
QT_MOC_LITERAL(0, 0, 25), // "QDeclarativeServiceFilter"
QT_MOC_LITERAL(1, 26, 18), // "serviceNameChanged"
QT_MOC_LITERAL(2, 45, 0), // ""
QT_MOC_LITERAL(3, 46, 3), // "arg"
QT_MOC_LITERAL(4, 50, 20), // "interfaceNameChanged"
QT_MOC_LITERAL(5, 71, 19), // "majorVersionChanged"
QT_MOC_LITERAL(6, 91, 19), // "minorVersionChanged"
QT_MOC_LITERAL(7, 111, 27), // "exactVersionMatchingChanged"
QT_MOC_LITERAL(8, 139, 34), // "monitorServiceRegistrationsCh..."
QT_MOC_LITERAL(9, 174, 26), // "serviceDescriptionsChanged"
QT_MOC_LITERAL(10, 201, 14), // "setServiceName"
QT_MOC_LITERAL(11, 216, 16), // "setInterfaceName"
QT_MOC_LITERAL(12, 233, 15), // "setMajorVersion"
QT_MOC_LITERAL(13, 249, 15), // "setMinorVersion"
QT_MOC_LITERAL(14, 265, 23), // "setExactVersionMatching"
QT_MOC_LITERAL(15, 289, 30), // "setMonitorServiceRegistrations"
QT_MOC_LITERAL(16, 320, 7), // "updates"
QT_MOC_LITERAL(17, 328, 17), // "updateServiceList"
QT_MOC_LITERAL(18, 346, 20), // "servicesAddedRemoved"
QT_MOC_LITERAL(19, 367, 11), // "serviceName"
QT_MOC_LITERAL(20, 379, 13), // "interfaceName"
QT_MOC_LITERAL(21, 393, 12), // "majorVersion"
QT_MOC_LITERAL(22, 406, 12), // "minorVersion"
QT_MOC_LITERAL(23, 419, 20), // "exactVersionMatching"
QT_MOC_LITERAL(24, 440, 27), // "monitorServiceRegistrations"
QT_MOC_LITERAL(25, 468, 19), // "serviceDescriptions"
QT_MOC_LITERAL(26, 488, 47) // "QQmlListProperty<QDeclarative..."

    },
    "QDeclarativeServiceFilter\0serviceNameChanged\0"
    "\0arg\0interfaceNameChanged\0majorVersionChanged\0"
    "minorVersionChanged\0exactVersionMatchingChanged\0"
    "monitorServiceRegistrationsChanged\0"
    "serviceDescriptionsChanged\0setServiceName\0"
    "setInterfaceName\0setMajorVersion\0"
    "setMinorVersion\0setExactVersionMatching\0"
    "setMonitorServiceRegistrations\0updates\0"
    "updateServiceList\0servicesAddedRemoved\0"
    "serviceName\0interfaceName\0majorVersion\0"
    "minorVersion\0exactVersionMatching\0"
    "monitorServiceRegistrations\0"
    "serviceDescriptions\0"
    "QQmlListProperty<QDeclarativeServiceDescriptor>"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_QDeclarativeServiceFilter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      15,   14, // methods
       7,  128, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       7,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   89,    2, 0x06 /* Public */,
       4,    1,   92,    2, 0x06 /* Public */,
       5,    1,   95,    2, 0x06 /* Public */,
       6,    1,   98,    2, 0x06 /* Public */,
       7,    1,  101,    2, 0x06 /* Public */,
       8,    1,  104,    2, 0x06 /* Public */,
       9,    0,  107,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    1,  108,    2, 0x0a /* Public */,
      11,    1,  111,    2, 0x0a /* Public */,
      12,    1,  114,    2, 0x0a /* Public */,
      13,    1,  117,    2, 0x0a /* Public */,
      14,    1,  120,    2, 0x0a /* Public */,
      15,    1,  123,    2, 0x0a /* Public */,
      17,    0,  126,    2, 0x08 /* Private */,
      18,    0,  127,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Bool,   16,
    QMetaType::Void,
    QMetaType::Void,

 // properties: name, type, flags
      19, QMetaType::QString, 0x00495103,
      20, QMetaType::QString, 0x00495103,
      21, QMetaType::Int, 0x00495103,
      22, QMetaType::Int, 0x00495103,
      23, QMetaType::Bool, 0x00495103,
      24, QMetaType::Bool, 0x00495103,
      25, 0x80000000 | 26, 0x00495009,

 // properties: notify_signal_id
       0,
       1,
       2,
       3,
       4,
       5,
       6,

       0        // eod
};

void QDeclarativeServiceFilter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<QDeclarativeServiceFilter *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->serviceNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->interfaceNameChanged((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->majorVersionChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->minorVersionChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->exactVersionMatchingChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->monitorServiceRegistrationsChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->serviceDescriptionsChanged(); break;
        case 7: _t->setServiceName((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 8: _t->setInterfaceName((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->setMajorVersion((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->setMinorVersion((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->setExactVersionMatching((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->setMonitorServiceRegistrations((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->updateServiceList(); break;
        case 14: _t->servicesAddedRemoved(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (QDeclarativeServiceFilter::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::serviceNameChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::interfaceNameChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::majorVersionChanged)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::minorVersionChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::exactVersionMatchingChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::monitorServiceRegistrationsChanged)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (QDeclarativeServiceFilter::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&QDeclarativeServiceFilter::serviceDescriptionsChanged)) {
                *result = 6;
                return;
            }
        }
    } else if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QQmlListProperty<QDeclarativeServiceDescriptor> >(); break;
        }
    }

#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<QDeclarativeServiceFilter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->serviceName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->interfaceName(); break;
        case 2: *reinterpret_cast< int*>(_v) = _t->majorVersion(); break;
        case 3: *reinterpret_cast< int*>(_v) = _t->minorVersion(); break;
        case 4: *reinterpret_cast< bool*>(_v) = _t->exactVersionMatching(); break;
        case 5: *reinterpret_cast< bool*>(_v) = _t->monitorServiceRegistrations(); break;
        case 6: *reinterpret_cast< QQmlListProperty<QDeclarativeServiceDescriptor>*>(_v) = _t->serviceDescriptions(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<QDeclarativeServiceFilter *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServiceName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setInterfaceName(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setMajorVersion(*reinterpret_cast< int*>(_v)); break;
        case 3: _t->setMinorVersion(*reinterpret_cast< int*>(_v)); break;
        case 4: _t->setExactVersionMatching(*reinterpret_cast< bool*>(_v)); break;
        case 5: _t->setMonitorServiceRegistrations(*reinterpret_cast< bool*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject QDeclarativeServiceFilter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_QDeclarativeServiceFilter.data,
    qt_meta_data_QDeclarativeServiceFilter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *QDeclarativeServiceFilter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QDeclarativeServiceFilter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_QDeclarativeServiceFilter.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    if (!strcmp(_clname, "org.qt-project.Qt.QQmlParserStatus"))
        return static_cast< QQmlParserStatus*>(this);
    return QObject::qt_metacast(_clname);
}

int QDeclarativeServiceFilter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 15;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 7;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 7;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void QDeclarativeServiceFilter::serviceNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void QDeclarativeServiceFilter::interfaceNameChanged(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void QDeclarativeServiceFilter::majorVersionChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void QDeclarativeServiceFilter::minorVersionChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void QDeclarativeServiceFilter::exactVersionMatchingChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void QDeclarativeServiceFilter::monitorServiceRegistrationsChanged(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void QDeclarativeServiceFilter::serviceDescriptionsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
